<?php
/**
 * Carbon functions and definitions
 *
 * @package Carbon
 */

/**
 * The current version of the theme.
 */
define( 'CARBON_VERSION', '1.0.8' );

/**
 * The suffix to use for scripts.
 */
if ( ( defined( 'SCRIPT_DEBUG' ) && true === SCRIPT_DEBUG ) ) {
	define( 'CARBON_SUFFIX', '' );
} else {
	define( 'CARBON_SUFFIX', '.min' );
}

/**
 * Set the content width based on the theme's design and stylesheet.
 */
if ( ! isset( $content_width ) ) {
	$content_width = 770; /* pixels */
}

if ( ! function_exists( 'carbon_setup' ) ) :
	/**
	 * Sets up theme defaults and registers support for various WordPress features.
	 *
	 * Note that this function is hooked into the after_setup_theme hook, which
	 * runs before the init hook. The init hook is too late for some features, such
	 * as indicating support for post thumbnails.
	 */
	function carbon_setup() {
		/*
		 * Make theme available for translation.
		 * Translations can be filed in the /languages/ directory.
		 * If you're building a theme based on Carbon, use a find and replace
		 * to change 'carbon' to the name of your theme in all the template files
		 */
		load_theme_textdomain( 'carbon', get_template_directory() . '/languages' );

		// Add default posts and comments RSS feed links to head.
		add_theme_support( 'automatic-feed-links' );

		/*
		 * Enable support for Post Thumbnails on posts and pages.
		 *
		 * @link http://codex.wordpress.org/Function_Reference/add_theme_support#Post_Thumbnails
		 */
		add_theme_support( 'post-thumbnails' );

		$custom_header_args = array(
			'width'              => 1170,
			'height'             => 600,
			'uploads'            => true,
			'flex-height'        => true,
			'flex-width'         => true,
			'default-text-color' => 'fff',
			'video'              => true,
		);
		add_theme_support( 'custom-header', $custom_header_args );

		// This theme uses wp_nav_menu() in one location.
		register_nav_menus(
			array(
				'primary' => esc_html__( 'Primary Menu', 'carbon' ),
				'footer'  => esc_html__( 'Footer Menu', 'carbon' ),
			)
		);

		/*
		 * Switch default core markup for search form, comment form, and comments
		 * to output valid HTML5.
		 */
		add_theme_support(
			'html5', array(
				'search-form',
				'comment-form',
				'comment-list',
				'gallery',
				'caption',
			)
		);

		// Set up the WordPress core custom background feature.
		add_theme_support(
			'custom-background', apply_filters(
				'carbon_custom_background_args', array(
					'default-color' => 'ffffff',
					'default-image' => '',
				)
			)
		);

		// Editor styles.
		add_editor_style( array( 'assets/css/editor-style.css' ) );

		// Add support for selective widget refresh.
		add_theme_support( 'customize-selective-refresh-widgets' );

		/**
		* Let WordPress manage the document title.
		* By adding theme support, we declare that this theme does not use a
		* hard-coded <title> tag in the document head, and expect WordPress to
		* provide it for us.
		*/
		add_theme_support( 'title-tag' );

		/**
		* Add Gutenberg Support.
		*/
		add_theme_support( 'wp-block-styles' );
		add_theme_support( 'align-wide' );
	}
endif; // Carbon_setup.
add_action( 'after_setup_theme', 'carbon_setup' );

if ( ! function_exists( 'carbon_image_sizes' ) ) :
	/**
	 * Register custom image sizes
	 *
	 * @since 1.0.0.
	 *
	 * @return void
	 */
	function carbon_image_sizes() {
		$defaults = array(
			'carbon_logo' => array( 500, 9999, false ),
		);

		$image_sizes = apply_filters( 'carbon_image_sizes', $defaults );

		foreach ( $image_sizes as $name => $prop ) {
			$crop = ( isset( $prop[2] ) ) ? $prop[2] : false;
			add_image_size( $name, $prop[0], $prop[1], $crop );
		}
	}
endif;

add_action( 'after_setup_theme', 'carbon_image_sizes' );

/**
 * Register widget area.
 *
 * @link http://codex.wordpress.org/Function_Reference/register_sidebar
 */
function carbon_widgets_init() {
	register_sidebar(
		array(
			'name'          => esc_html__( 'Main Sidebar', 'carbon' ),
			'id'            => 'sidebar-1',
			'description'   => '',
			'before_widget' => '<aside id="%1$s" class="widget %2$s">',
			'after_widget'  => '</aside>',
			'before_title'  => '<h3 class="widget-title">',
			'after_title'   => '</h3>',
		)
	);
	register_sidebar(
		array(
			'name'          => esc_html__( 'Single Posts Sidebar', 'carbon' ),
			'id'            => 'sidebar-2',
			'description'   => esc_html__( 'An optional widget area for single posts.', 'carbon' ),
			'before_widget' => '<aside id="%1$s" class="widget %2$s">',
			'after_widget'  => '</aside>',
			'before_title'  => '<h3 class="widget-title">',
			'after_title'   => '</h3>',
		)
	);
	register_sidebar(
		array(
			'name'          => esc_html__( 'Single Page Sidebar', 'carbon' ),
			'id'            => 'sidebar-3',
			'description'   => esc_html__( 'An optional widget area for single pages.', 'carbon' ),
			'before_widget' => '<aside id="%1$s" class="widget %2$s">',
			'after_widget'  => '</aside>',
			'before_title'  => '<h3 class="widget-title">',
			'after_title'   => '</h3>',
		)
	);
	register_sidebar(
		array(
			'name'          => esc_html__( 'Archive Sidebar', 'carbon' ),
			'id'            => 'sidebar-4',
			'description'   => esc_html__( 'An optional widget area for archives.', 'carbon' ),
			'before_widget' => '<aside id="%1$s" class="widget %2$s">',
			'after_widget'  => '</aside>',
			'before_title'  => '<h3 class="widget-title">',
			'after_title'   => '</h3>',
		)
	);
}
add_action( 'widgets_init', 'carbon_widgets_init' );

/**
 * Enqueue Gutenberg Styles & Scripts.
 */
function carbon_block_editor_styles() {

	$style_dependencies = array();

	/**
	 * Getting & Setting Google Fonts
	 */
	// Google fonts.
	$google_request = stag_get_google_font_uri();
	if ( '' !== $google_request ) {
		// Enqueue the fonts.
		wp_enqueue_style(
			'carbon-google-fonts',
			$google_request,
			$style_dependencies,
			CARBON_VERSION
		);
		$style_dependencies[] = 'carbon-google-fonts';
	}

	// Add Genericons, used in the main stylesheet.
	wp_enqueue_style( 'genericons', get_template_directory_uri() . '/assets/css/genericons/genericons.css', $style_dependencies, '3.4.1' );
	$style_dependencies[] = 'genericons';

	// Block Editor Styles.
	wp_enqueue_style( 'carbon-block-editor-style', get_template_directory_uri() . '/assets/css/block-editor-style.css', $style_dependencies, CARBON_VERSION );

	$font_header       = stag_theme_mod( 'fonts', 'font-header' );
	$font_header_stack = stag_get_font_stack( $font_header );

	$font_body       = stag_theme_mod( 'fonts', 'font-body' );
	$font_body_stack = stag_get_font_stack( $font_body );

	$accent_color = stag_theme_mod( 'colors', 'accent' );

	wp_add_inline_style(
		'carbon-block-editor-style', "
		.edit-post-layout__content{
			--accent-color: {$accent_color};
		}
		.edit-post-visual-editor {
			--font-body: {$font_body_stack};
			--font-header: {$font_header_stack};
		}"
	);
}
add_action( 'enqueue_block_editor_assets', 'carbon_block_editor_styles' );


/**
 * Enqueue scripts and styles.
 */
function carbon_scripts() {
	$style_dependencies = array();

	// Google fonts.
	if ( '' !== $google_request = stag_get_google_font_uri() ) {
		// Enqueue the fonts.
		wp_enqueue_style(
			'carbon-google-fonts',
			$google_request,
			$style_dependencies,
			CARBON_VERSION
		);
		$style_dependencies[] = 'carbon-google-fonts';
	}

	// Add Genericons, used in the main stylesheet.
	wp_enqueue_style( 'genericons', get_template_directory_uri() . '/assets/css/genericons/genericons.css', $style_dependencies, '3.4.1' );
	$style_dependencies[] = 'genericons';

	wp_enqueue_style( 'carbon-style', get_stylesheet_uri(), $style_dependencies, CARBON_VERSION );

	wp_enqueue_script( 'carbon-frontend', get_template_directory_uri() . '/assets/js/frontend' . CARBON_SUFFIX . '.js', array( 'jquery' ), CARBON_VERSION, true );

	if ( is_singular() && comments_open() && get_option( 'thread_comments' ) ) {
		wp_enqueue_script( 'comment-reply' );
	}
}
add_action( 'wp_enqueue_scripts', 'carbon_scripts' );

/**
 * Register the required plugins for this theme.
 *
 * @since 1.0.0
 *
 * @return void
 */
function carbon_required_plugins() {
	$plugins = array(
		array(
			'name'     => 'Jetpack',
			'slug'     => 'jetpack',
			'required' => true,
		),
		array(
			'name'     => 'StagTools',
			'slug'     => 'stagtools',
			'required' => true,
		),
		array(
			'name'         => 'Envato WordPress Toolkit',
			'slug'         => 'envato-wordpress-toolkit',
			'source'       => 'https://github.com/envato/envato-wordpress-toolkit/archive/master.zip',
			'required'     => false,
			'version'      => '1.7.3',
			'external_url' => 'https://github.com/envato/envato-wordpress-toolkit/',
		),
	);

	tgmpa( $plugins );
}
add_action( 'tgmpa_register', 'carbon_required_plugins' );

// Customizer additions.
require get_template_directory() . '/inc/customizer.php';
require get_template_directory() . '/stag-customizer/stag-customizer.php';

// Include theme partials: widgets, metaboxes and rest.
require get_template_directory() . '/inc/init.php';

/**
 * Load auto updater
 *
 * @since 1.0.4.
 */
require get_template_directory() . '/inc/updater/theme-updater.php';
