<?php
/**
 * Custom template tags for this theme.
 *
 * Eventually, some of the functionality here could be replaced by core features.
 *
 * @package Carbon
 */

if ( ! function_exists( 'carbon_paging_nav' ) ) :
/**
 * Display navigation to next/previous set of posts when applicable.
 */
function carbon_paging_nav() {
	// Don't print empty markup if there's only one page.
	if ( $GLOBALS['wp_query']->max_num_pages < 2 ) {
		return;
	}

	?>
	<nav class="navigation paging-navigation" role="navigation">
		<h1 class="screen-reader-text"><?php esc_html_e( 'Posts navigation', 'carbon' ); ?></h1>
		<div class="nav-links">

			<?php if ( get_next_posts_link() ) : ?>
			<div class="nav-previous"><?php next_posts_link( _x( '<span class="meta-nav">&larr;</span> Older posts', 'Older post link', 'carbon' ) ); ?></div>
			<?php endif; ?>

			<?php if ( get_previous_posts_link() ) : ?>
			<div class="nav-next"><?php previous_posts_link( _x( 'Newer posts <span class="meta-nav">&rarr;</span>', 'Next post link', 'carbon' ) ); ?></div>
			<?php endif; ?>

		</div><!-- .nav-links -->
	</nav><!-- .navigation -->
	<?php
}
endif;

if ( ! function_exists( 'carbon_post_nav' ) ) :
/**
 * Display navigation to next/previous post when applicable.
 */
function carbon_post_nav() {
	// Don't print empty markup if there's nowhere to navigate.
	$previous = ( is_attachment() ) ? get_post( get_post()->post_parent ) : get_adjacent_post( false, '', true );
	$next     = get_adjacent_post( false, '', false );

	if ( ! $next && ! $previous ) {
		return;
	}
	?>
	<nav class="navigation post-navigation" role="navigation">
		<h1 class="screen-reader-text"><?php esc_html_e( 'Post navigation', 'carbon' ); ?></h1>
		<div class="nav-links">
			<?php
				previous_post_link( '<div class="nav-previous">%link</div>', _x( '<span class="meta-nav">&larr;</span>&nbsp;%title', 'Previous post link', 'carbon' ) );
				next_post_link( '<div class="nav-next">%link</div>', _x( '%title&nbsp;<span class="meta-nav">&rarr;</span>', 'Next post link', 'carbon' ) );
			?>
		</div><!-- .nav-links -->
	</nav><!-- .navigation -->
	<?php
}
endif;

if ( ! function_exists( 'carbon_posted_on' ) ) :
/**
 * Prints HTML with meta information for the current post-date/time and author.
 */
function carbon_posted_on() {
	$time_string = '<time class="entry-date published updated" datetime="%1$s">%2$s</time>';
	if ( get_the_time( 'U' ) !== get_the_modified_time( 'U' ) ) {
		$time_string = '<time class="entry-date published" datetime="%1$s">%2$s</time><time class="updated" datetime="%3$s">%4$s</time>';
	}

	$time_string = sprintf( $time_string,
		esc_attr( get_the_date( 'c' ) ),
		esc_html( get_the_date() ),
		esc_attr( get_the_modified_date( 'c' ) ),
		esc_html( get_the_modified_date() )
	);

	$posted_on = sprintf(
		_x( 'Posted on %s', 'post date', 'carbon' ),
		'<a href="' . esc_url( get_permalink() ) . '" rel="bookmark">' . $time_string . '</a>'
	);

	$author = get_the_author();

	if ( empty( $author ) ) {
		global $wp_query;
		$authordata = $GLOBALS['authordata'] = get_userdata( $wp_query->post->post_author );
		$author = $authordata->display_name;
	}

	$byline = sprintf(
		_x( 'by %s', 'post author', 'carbon' ),
		'<span class="author vcard"><a class="url fn n" href="' . esc_url( get_author_posts_url( get_the_author_meta( 'ID' ) ) ) . '">' . esc_html( $author ) . '</a></span>'
	);

	echo '<span class="posted-on">' . $posted_on . '</span><span class="byline"> ' . $byline . '</span>';

	// Hide category and tag text for pages.
	if ( 'post' == get_post_type() ) {
		/* translators: used between list items, there is a space after the comma */
		$categories_list = get_the_category_list( esc_html__( ', ', 'carbon' ) );
		if ( $categories_list && carbon_categorized_blog() ) {
			printf( '<span class="cat-links">' . esc_html__( ', In %1$s', 'carbon' ) . '</span>', $categories_list );
		}
	}

	if ( $reading_time = carbon_post_reading_time() ) {
		echo '<span class="reading-time">' . $reading_time . '</span>';
	}
}
endif;

/**
 * Returns true if a blog has more than 1 category.
 *
 * @return bool
 */
function carbon_categorized_blog() {
	if ( false === ( $all_the_cool_cats = get_transient( 'carbon_categories' ) ) ) {
		// Create an array of all the categories that are attached to posts.
		$all_the_cool_cats = get_categories( array(
			'fields'     => 'ids',
			'hide_empty' => 1,

			// We only need to know if there is more than one category.
			'number'     => 2,
		) );

		// Count the number of categories that are attached to the posts.
		$all_the_cool_cats = count( $all_the_cool_cats );

		set_transient( 'carbon_categories', $all_the_cool_cats );
	}

	if ( $all_the_cool_cats > 1 ) {
		// This blog has more than 1 category so carbon_categorized_blog should return true.
		return true;
	} else {
		// This blog has only 1 category so carbon_categorized_blog should return false.
		return false;
	}
}

/**
 * Flush out the transients used in carbon_categorized_blog.
 */
function carbon_category_transient_flusher() {
	// Like, beat it. Dig?
	delete_transient( 'carbon_categories' );
}
add_action( 'edit_category', 'carbon_category_transient_flusher' );
add_action( 'save_post',     'carbon_category_transient_flusher' );

if ( ! function_exists( 'carbon_has_logo' ) ) :
/**
 * Determine if a site logo is assigned or not.
 *
 * @since 1.0.0.
 *
 * @return bool
 */
function carbon_has_logo() {
	if ( function_exists( 'jetpack_has_site_logo' ) ) {
		return jetpack_has_site_logo();
	} else {
		return false;
	}
}
endif;

if ( ! function_exists( 'carbon_get_view' ) ) :
/**
 * Determine the current view.
 *
 * For use with view-related theme options.
 *
 * @since  1.0.0.
 *
 * @return string    The string representing the current view.
 */
function carbon_get_view() {
	// Post types
	$post_types = get_post_types(
		array(
			'public' => true,
			'_builtin' => false,
		)
	);
	$post_types[] = 'post';

	// Post parent
	$parent_post_type = '';
	if ( is_attachment() ) {
		$post_parent      = get_post()->post_parent;
		$parent_post_type = get_post_type( $post_parent );
	}

	$view = 'post';

	// Blog
	if ( is_home() ) {
		$view = 'blog';
	}
	// Archives
	else if ( is_archive() ) {
		$view = 'archive';
	}
	// Search results
	else if ( is_search() ) {
		$view = 'search';
	}
	// Posts and public custom post types
	else if ( is_singular( $post_types ) || ( is_attachment() && in_array( $parent_post_type, $post_types ) ) ) {
		$view = 'post';
	}
	// Pages
	else if ( is_page() || ( is_attachment() && 'page' === $parent_post_type ) ) {
		$view = 'page';
	}

	/**
	 * Allow developers to dynamically change the view.
	 *
	 * @since 1.0.0.
	 *
	 * @param string    $view                The view name.
	 * @param string    $parent_post_type    The post type for the parent post of the current post.
	 */
	return apply_filters( 'carbon_get_view', $view, $parent_post_type );
}
endif;

if ( ! function_exists( 'carbon_contextual_view' ) ) :
/**
 * Display optional elements depending on site view.
 *
 * @since 1.0.0.
 *
 * @return mixed
 */
function carbon_contextual_view() {

	if ( is_archive() ) :
		get_template_part( 'partials/sidebar', 'archive' );

	elseif ( is_search() ) :
		get_template_part( 'partials/sidebar', 'search' );

	else :
		return false;
	endif;

}
endif;

if ( ! function_exists( 'carbon_post_reading_time' ) ) :
/**
 * Calculate post reading time.
 *
 * Cache the post reading time under post meta for better performance.
 *
 * @uses stag_get_post_meta()
 * @uses stag_update_post_meta()
 * @since 1.0.0.
 * @param int $post_id Post ID of which to calculate the post reading time.
 * @return mixed
 */
function carbon_post_reading_time( $post_id = null ) {

	if ( false === stag_theme_mod( 'theme_options', 'show-reading-time' ) ) return false;

	if ( 0 === absint( $post_id ) ) {
		$post_id = get_the_ID();
	}

	// Use cached value if available.
	$char_count = stag_get_post_meta( 'cache', $post_id, 'char-count' );
	if ( false !== $char_count ) {
		$char_count = absint( $char_count );
	} else {
		$content = wp_strip_all_tags( strip_shortcodes( get_post( $post_id )->post_content ), true );

		/**
		 * Use - count( preg_split( '/\s+/', $content ) )
		 * for compatibility with cyrillic or other characters which are not assumed words.
		 *
		 * @since 1.1.0
		 */
		$char_count = apply_filters( 'post_words_count', str_word_count( $content ) );

		// Cache the value.
		stag_update_post_meta( 'cache', $post_id, 'char-count', $char_count );
	}

	$wpm = apply_filters( 'carbon_words_per_minute', 200 );

	// Get Estimated time.
	$minutes = floor( $char_count / $wpm );
	$seconds = floor( ( $char_count / ( $wpm / 60 ) ) - ( $minutes * 60 ) );

	// If less than a minute.
	if ( $minutes < 1 ) {
		$estimated_time = esc_html__( '1 minute read', 'carbon' );
	}

	// If more than a minute.
	if ( $minutes >= 1 ) {

		if ( $seconds > 30 ) {
			$minutes++;
		}

		/* translators: %d = minute count */
		$estimated_time = sprintf( esc_html__( '%d minute read', 'carbon' ), $minutes );
	}

	return $estimated_time;
}
endif;
