<?php
/**
 * @package Stag_Customizer
 * @author Ram Ratan Maurya (Codestag)
 */

/**
 * Stag Customizer.
 *
 * @version 1.0
 */
final class Stag_Customizer {
	/**
	 * @var Stag_Customizer The single instance of Stag_Customizer class.
	 */
	protected static $_instance = null;

	/**
	 * Stag_Customizer Version.
	 *
	 * @var string
	 */
	public $version = '1.0';

	/**
	 * Main Stag_Customizer class.
	 *
	 * Ensure only one instance of Stag_Customizer is loaded or can be loaded.
	 *
	 * @return Stag_Customizer - Main instance.
	 */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
			self::$_instance->setup_constants();
			self::$_instance->includes();
		}
		return self::$_instance;
	}

	/**
	 * Stag Customizer constructor.
	 *
	 * @access public
	 * @return Stag_Customizer
	 */
	public function __construct() {
		// Filters that allow shortcodes in Text widgets
		add_filter( 'widget_text', 'shortcode_unautop' );
		add_filter( 'widget_text', 'do_shortcode' );

		add_action( 'after_setup_theme', array( $this, 'customizer_init' ) );
		add_action( 'customize_preview_init', array( $this, 'customizer_preview_js' ) );
	}

	public function customizer_init() {
		add_action( 'customize_register', array( $this, 'customize_register_settings' ) );
		add_action( 'customize_register', array( $this, 'customize_register_transport' ) );
	}

	/**
	 * Setup theme constants.
	 *
	 * @access private
	 * @since 1.0
	 * @return void
	 */
	private function setup_constants() {
		if ( function_exists( 'wp_get_theme' ) ) {

			if ( is_child_theme() ) {
				$temp_obj  = wp_get_theme();
				$theme_obj = wp_get_theme( $temp_obj->get( 'Template' ) );
			} else {
				$theme_obj = wp_get_theme();
			}

			$theme_version = $theme_obj->get( 'Version' );
			$theme_name    = $theme_obj->get( 'Name' );
		}

		define( 'STAG_THEME_NAME', $theme_name );
		define( 'STAG_THEME_VERSION', $theme_version );

		/**
		 * The suffix to use for scripts.
		 */
		if ( ( defined( 'SCRIPT_DEBUG' ) && true === SCRIPT_DEBUG ) ) {
			define( 'STAG_SCRIPT_SUFFIX', '' );
		} else {
			define( 'STAG_SCRIPT_SUFFIX', '.min' );
		}
	}

	/**
	 * Binds JS handlers to make Theme Customizer preview reload changes asynchronously.
	 *
	 * @since 1.0
	 * @return void
	 */
	public function customizer_preview_js() {
		wp_enqueue_script( 'stag-customizer-preview', get_template_directory_uri() . '/stag-customizer/js/customizer-preview.js', array( 'jquery', 'customize-preview' ), CARBON_VERSION, true );
	}

	/**
	 * Include required files.
	 *
	 * @access private
	 * @since 1.0
	 * @return void
	 */
	private function includes() {
		// Common includes
		global $pagenow;

		$path = get_template_directory() . '/stag-customizer/';

		require_once $path . 'helpers-tgmpa.php';
		require_once $path . 'helpers-widget.php';
		require_once $path . 'helpers-logo.php';
		require_once $path . 'google-fonts.php';
		require_once $path . 'helpers-fonts.php';
		require_once $path . 'helpers-css.php';
		require_once $path . 'controls.php';
	}

	/**
	 * Register settings.
	 *
	 * Take the final list of theme mods, and register all the settings,
	 * and add all of the proper controls.
	 *
	 * If the type is one of the default supported ones, add it normally. Otherwise
	 * Use the type to create a new instance of that control type.
	 *
	 * @since 1.0
	 *
	 * @param WP_Customize_Manager $wp_customize Theme Customizer object.
	 * @return void
	 */
	public function customize_register_settings( $wp_customize ) {
		$mods = stag_get_theme_mods();

		foreach ( $mods as $section => $settings ) {
			foreach ( $settings as $key => $setting ) {
				$wp_customize->add_setting(
					$key, array(
						'default'           => stag_theme_mod( $section, $key, true ),
						'sanitize_callback' => isset( $setting['sanitize_callback'] ) ? $setting['sanitize_callback'] : null,
					)
				);

				$type = $setting['type'];

				if ( in_array( $type, array( 'text', 'checkbox', 'radio', 'select', 'dropdown-pages', 'url', 'textarea', 'range' ) ) ) {
					$wp_customize->add_control(
						$key, array(
							'label'       => $setting['title'],
							'section'     => $section,
							'settings'    => $key,
							'type'        => $type,
							'choices'     => isset( $setting['choices'] ) ? $setting['choices'] : null,
							'priority'    => isset( $setting['priority'] ) ? $setting['priority'] : null,
							'description' => isset( $setting['description'] ) ? $setting['description'] : null,
							'input_attrs' => isset( $setting['input_attrs'] ) ? $setting['input_attrs'] : null,
						)
					);
				} else {
					$wp_customize->add_control(
						new $type(
							$wp_customize, $key, array(
								'label'       => isset( $setting['title'] ) ? $setting['title'] : null,
								'section'     => $section,
								'settings'    => $key,
								'choices'     => isset( $setting['choices'] ) ? $setting['choices'] : null,
								'priority'    => isset( $setting['priority'] ) ? $setting['priority'] : null,
								'type'        => isset( $setting['option'] ) ? $setting['option'] : null,
								'description' => isset( $setting['description'] ) ? $setting['description'] : null,
							)
						)
					);
				}
			}
		}

		do_action( 'stag_customize_regiser_settings', $wp_customize );

		return $wp_customize;
	}

	/**
	 * Add postMessage support for all default fields, as well
	 * as the site title and desceription for the Theme Customizer.
	 *
	 * @since 1.0
	 *
	 * @param WP_Customize_Manager $wp_customize Theme Customizer object.
	 * @return void
	 */
	function customize_register_transport( $wp_customize ) {
		$built_in = array(
			'blogname'         => '',
			'blogdescription'  => '',
			'header_textcolor' => '',
		);
		$stag     = stag_get_theme_mods( array( 'keys_only' => true ) );

		$transport = array_merge( $built_in, $stag );

		foreach ( $transport as $key => $default ) {

			$transport = ( '' != $default ) ? $default : 'postMessage';

			$wp_customize->get_setting( $key )->transport = $transport;
		}
	}
}

function _sc() {
	return Stag_Customizer::instance();
}

$GLOBALS['stag_customizer'] = _sc();


/**
 * Get Theme Mod
 *
 * Instead of options, customizations are stored/accessed via Theme Mods
 * (which are still technically settings). This wrapper provides a way to
 * check for an existing mod, or load a default in its place.
 *
 * @since 1.0
 *
 * @param string $key The key of the theme mod to check. Prefixed with 'stag_'
 * @return mixed The theme modification setting
 */
function stag_theme_mod( $section, $key, $_default = false ) {
	$mods = stag_get_theme_mods();

	$default = $mods[ $section ][ $key ]['default'];

	if ( $_default )
		$mod  = $default;
	else $mod = get_theme_mod( $key, $default );

	return apply_filters( 'stag_theme_mod_' . $key, $mod );
}

if ( ! function_exists( 'stag_get_post_meta' ) ) :
/**
 * Get a specific value from one of the two post meta arrays.
 *
 * Posts in Ink store meta data in two different arrays. The "cache" array
 * is for data that is generated during a page load that can be stored and
 * retrieved later for better performance. The "settings" array is for user-
 * specified data pertaining to the post.
 *
 * @since 1.0.
 *
 * @param  string $type    Which meta array. Either 'cache' or 'settings'.
 * @param  int    $post_id The post id.
 * @param  string $key     The array key to target.
 * @return mixed|bool      The stored value, or false if it doesn't exist.
 */
function stag_get_post_meta( $type, $post_id, $key ) {
	$post_meta = get_post_meta( $post_id, '_stag-post-' . $type, true );

	if ( isset( $post_meta[ $key ] ) ) {
		return $post_meta[ $key ];
	} else {
		return false;
	}
}
endif;

if ( ! function_exists( 'stag_update_post_meta' ) ) :
/**
 * Update or remove an item from one of the two post meta arrays.
 *
 * Posts in Ink store meta data in two different arrays. The "cache" array
 * is for data that is generated during a page load that can be stored and
 * retrieved later for better performance. The "settings" array is for user-
 * specified data pertaining to the post.
 *
 * @since 1.0.
 *
 * @param string $type    Which meta array. Either 'cache' or 'settings'.
 * @param int    $post_id The post id.
 * @param string $key     The array key to target.
 * @param mixed  $value   The value to set.
 */
function stag_update_post_meta( $type, $post_id, $key, $value ) {
	if ( ! in_array( $type, array( 'cache', 'settings' ) ) ) {
		return;
	}

	$post_meta = get_post_meta( $post_id, '_stag-post-' . $type, true );

	// If value is null, remove the array item.
	if ( null === $value && isset( $post_meta[ $key ] ) ) {
		unset( $post_meta[ $key ] );
	} else { // Otherwise set/add the array item.
		$post_meta[ $key ] = $value;
	}

	update_post_meta( $post_id, '_stag-post-' . $type, $post_meta );
}
endif;

if ( ! function_exists( 'stag_add_customizations' ) ) :
/**
 * Make sure the 'stag_css' action only runs once.
 *
 * @since  1.0.1.
 *
 * @return void
 */
function stag_add_customizations() {
	do_action( 'stag_css' );
}
endif;

add_action( 'admin_init', 'stag_add_customizations' );

if ( ! function_exists( 'stag_display_customizations' ) ) :
/**
 * Generates the style tag and CSS needed for the theme options.
 *
 * By using the "stag_css" filter, different components can print CSS in the header. It is organized this way to
 * ensure that there is only one "style" tag and not a proliferation of them.
 *
 * @since  1.0.1.
 *
 * @return void
 */
function stag_display_customizations() {

	// if ( false === ( $css = get_transient( 'stag_css' ) ) || is_customize_preview() ) {
	do_action( 'stag_css' );

	// Get the rules
	$css = stag_get_css()->build();

	// if ( ! is_customize_preview() ) {
	// set_transient( 'stag_css', $css, WEEK_IN_SECONDS );
	// }
	// }
	if ( ! empty( $css ) ) {
		echo "\n<!-- Begin Stag Custom CSS -->\n<style type=\"text/css\" id=\"stag-custom-css\">\n";
		echo $css;
		echo "\n</style>\n<!-- End Stag Custom CSS -->\n";
	}
}
endif;

add_action( 'wp_head', 'stag_display_customizations', 11 );

if ( ! function_exists( 'stag_ajax_display_customizations' ) ) :
/**
 * Generates the theme option CSS as an Ajax response
 *
 * @since  1.0.0.
 *
 * @return void
 */
function stag_ajax_display_customizations() {
	// Make sure this is an Ajax request
	if ( ! defined( 'DOING_AJAX' ) || true !== DOING_AJAX ) {
		return;
	}

	// Set the content type
	header( 'Content-Type: text/css' );

	// Echo the rules
	echo stag_get_css()->build();

	// End the Ajax response
	die();
}
endif;

add_action( 'wp_ajax_stag-css', 'stag_ajax_display_customizations' );

if ( ! function_exists( 'stag_mce_css' ) ) :
/**
 * Make sure theme option CSS is added to TinyMCE last, to override other styles.
 *
 * @since  1.0.0.
 *
 * @param  string $stylesheets    List of stylesheets added to TinyMCE.
 * @return string                    Modified list of stylesheets.
 */
function stag_mce_css( $stylesheets ) {
	if ( stag_get_css()->build() ) {
		$stylesheets .= ',' . add_query_arg( 'action', 'stag-css', admin_url( 'admin-ajax.php' ) );
	}

	return $stylesheets;
}
endif;

add_filter( 'mce_css', 'stag_mce_css', 99 );

if ( ! function_exists( 'stag_refresh_custom_css' ) ) :
/**
 * Refresh the custom css cache after the customizer is saved.
 *
 * @since  1.0.0.
 *
 * @return void
 */
function stag_refresh_custom_css() {
	delete_transient( 'stag_css' );
}
endif;

add_action( 'customize_save_after', 'stag_refresh_custom_css' );
add_action( 'customize_preview_init', 'stag_refresh_custom_css' );

if ( ! function_exists( 'is_customize_preview' ) ) :
/**
 * Whether the site is being previewed in the Customizer.
 *
 * @return bool True if the site is being previewed in the Customizer, false otherwise.
 */
function is_customize_preview() {
	global $wp_customize;

	return is_a( $wp_customize, 'WP_Customize_Manager' ) && $wp_customize->is_preview();
}
endif;
